from .charts import *
"""
Dictionary of visualization charts 
"""
chartdict = {
    'imagegrid': ImageGrid,
    'timeline': Timeline,
    'graph': Graph,
    'dimensions': Dimensions,
    'map': Map,
    'tree': Tree,
    'wordcloud': WordCloud,
    'linechart': LineChart,
    'barchart': BarChart,
    'histogram': Histogram,
    'densityplot': DensityPlot,
    'treemap': TreeMap,
    'sunburstchart': SunBurstChart,
    'heatmap': HeatMap,
    'piechart': PieChart,
    'donutchart': DonutChart,
    'boxplot': BoxPlot,
    'violinplot': ViolinPlot,
    'areachart': AreaChart,
    'stackedareachart': StackedAreaChart,
    'scatterchart': ScatterChart,
    'bubblechart': BubbleChart,
    'table': Table
}